@@ -1,5 +1,8 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
+import datetime
+from collections import defaultdict
+
 from trytond.model import ModelView, ModelSQL, fields
 from trytond.pyson import Eval
 from trytond.pool import Pool, PoolMeta
@@ -7,9 +10,9 @@ from trytond.transaction import Transaction
 from trytond.modules.stock import StockMixin
 
 __all__ = ['Lot', 'LotType', 'Move', 'ShipmentIn', 'ShipmentOut',
-    'ShipmentOutReturn',
+    'LotByLocationContext', 'ShipmentOutReturn',
     'Period', 'PeriodCacheLot',
-    'Inventory', 'InventoryLine']
+    'Inventory', 'InventoryLine', 'LotByLocationContext']
 
 
 class Lot(ModelSQL, ModelView, StockMixin):
@@ -37,6 +40,29 @@ class Lot(ModelSQL, ModelView, StockMixin):
             grouping=('product', 'lot'))
 
 
+class LotByLocationContext(ModelView):
+    'Lot by Location'
+    __name__ = 'stock.lots_by_location.context'
+    forecast_date = fields.Date(
+        'At Date', help=('Allow to compute expected '
+            'stock quantities for this date.\n'
+            '* An empty value is an infinite date in the future.\n'
+            '* A date in the past will provide historical values.'))
+    stock_date_end = fields.Function(fields.Date('At Date'),
+        'on_change_with_stock_date_end')
+
+    @staticmethod
+    def default_forecast_date():
+        Date = Pool().get('ir.date')
+        return Date.today()
+
+    @fields.depends('forecast_date')
+    def on_change_with_stock_date_end(self, name=None):
+        if self.forecast_date is None:
+            return datetime.date.max
+        return self.forecast_date
+
+
 class LotType(ModelSQL, ModelView):
     "Stock Lot Type"
     __name__ = 'stock.lot.type'
@@ -102,11 +128,11 @@ class ShipmentOut:
         Move = pool.get('stock.move')
         super(ShipmentOut, cls)._sync_inventory_to_outgoing(
             shipments, create=create, write=write)
+        to_write = []
         for shipment in shipments:
-            outgoing_by_product = {}
+            outgoing_by_product = defaultdict(list)
             for move in shipment.outgoing_moves:
-                outgoing_by_product.setdefault(move.product.id,
-                    []).append(move)
+                outgoing_by_product[move.product.id].append(move)
             for move in shipment.inventory_moves:
                 if not move.lot:
                     continue
@@ -118,19 +144,19 @@ class ShipmentOut:
                     out_quantity = Uom.compute_qty(out_move.uom,
                         out_move.quantity, out_move.product.default_uom,
                         round=False)
+                    values = {}
                     if quantity < out_quantity:
                         outgoing_moves.extend(Move.copy([out_move], default={
                                     'quantity': out_move.uom.round(
                                         out_quantity - quantity),
                                     }))
-                        Move.write([out_move], {
-                                'quantity': out_move.uom.round(quantity),
-                                })
-                    Move.write([out_move], {
-                            'lot': move.lot.id,
-                            })
+                        values['quantity'] = out_move.uom.round(quantity)
+                    values['lot'] = move.lot.id
+                    to_write.extend(([out_move], values))
                     quantity -= out_quantity
                 assert move.uom.round(quantity) <= 0
+        if to_write:
+            Move.write(*to_write)
 
 
 class ShipmentOutReturn:
@@ -199,7 +225,10 @@ class InventoryLine:
         domain=[
             ('product', '=', Eval('product')),
             ],
-        depends=['product'])
+        states={
+            'readonly': Eval('inventory_state') != 'draft',
+            },
+        depends=['product', 'inventory_state'])
 
     @classmethod
     def __setup__(cls):
